@@ -51,15 +51,17 @@ class Author:
 
 @dataclass(unsafe_hash=True)
 class Dataset:
-    doi_dataset: str = None
+    doi: str = None
     link: str = None
     name: str = None
     description: str = None
     license: str = None
     author: str = None
+    path: str = None
 
 @dataclass(unsafe_hash=True)
 class Software:
+    metadata: dict = None
     link: str = None
     name: str = None
     description: str = None
@@ -79,6 +81,19 @@ class Bibliography_entry:
 
 ######################## FUNCTIONS #################################
 
+class HiddenPrints:
+    def __enter__(self):
+        self._original_stdout = sys.stdout
+        self._original_stderr = sys.stderr
+        sys.stdout = open(os.devnull, 'w')
+        sys.stderr = open(os.devnull, 'w')
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        sys.stdout.close()
+        sys.stderr.close()
+        sys.stdout = self._original_stdout
+        sys.stderr = self._original_stderr
+
 
 import yaml 
 from yaml.loader import SafeLoader
@@ -87,9 +102,15 @@ import os
 import sys
 from . import req_orcid
 from . import req_doi
-import somef.cli
+from somef.somef_cli import cli_get_data
 from . import properties as p
 import json
+from shutil import copyfile
+from soca.commands.portal.metadata import metadata as soca_metadata
+import warnings
+warnings.filterwarnings("ignore")
+import metadata_parser
+
 
 
 def load_jsonld(input_jsonld):
@@ -110,7 +131,8 @@ def load_jsonld(input_jsonld):
             index_html = entry["@id"]
             break
     
-    type = root["DataType"]
+    type = str(root["DataType"]).lower()
+    
 
     if type == "project":
 
@@ -140,11 +162,13 @@ def load_jsonld(input_jsonld):
 
 def load_yaml(input_yaml):
 
+    global output_directory_datafolder
+
     output_directory_datafolder = Path(p.output_directory, str(Path(input_yaml).stem))
 
     if not os.path.exists(output_directory_datafolder):
         os.makedirs(output_directory_datafolder)
-        print(f"Creating output diretory {output_directory_datafolder}")
+        print(f"Creating output directory {output_directory_datafolder}")
 
     # Create htaccess file
     from . import htaccess
@@ -154,15 +178,17 @@ def load_yaml(input_yaml):
     images_output_path = Path(output_directory_datafolder, p.input_to_vocab["images"])
     if not os.path.exists(images_output_path):
         os.makedirs(images_output_path)
-        print(f"Creating images diretory {output_directory_datafolder}/images")
+        print(f"Creating images directory {output_directory_datafolder}/images")
 
     # Open input.yaml and parse it
     with open(Path(input_yaml)) as file:
         data = yaml.load(file, Loader=SafeLoader)
     
-    print(f"Parsing and fetching info from {input_yaml}...\n")
+    print(f"Parsing and fetching info from {input_yaml}\n")
 
     type = _safe(p.input_to_vocab["type"], data)
+    if type:
+        del data["type"]
      
     if type not in ["paper", "project"]:
         print("""ERROR: Is required to specify a type field in the input yaml. Options: 'paper', 'project'.
@@ -171,20 +197,24 @@ def load_yaml(input_yaml):
         exit()
     
     if type == "paper":
-        data = init_paper(p.input_to_vocab, data)
+        init_data = init_paper(p.input_to_vocab, data)
         
     if type == "project":
-        data = init_project(p.input_to_vocab, data)
+        init_data = init_project(p.input_to_vocab, data)
 
     # Init data meta properties
-    data.output_html = Path(output_directory_datafolder, p.properties["output_html"])
-    data.output_html_help = Path(output_directory_datafolder, p.properties["output_html_help"])
-    data.output_jsonld = Path(output_directory_datafolder, p.properties["output_jsonld"])
-    data.output_directory_datafolder = output_directory_datafolder
-    data.type = type
-    data.yaml_file = input_yaml
+    init_data.output_html = Path(output_directory_datafolder, p.properties["output_html"])
+    init_data.output_html_help = Path(output_directory_datafolder, p.properties["output_html_help"])
+    init_data.output_jsonld = Path(output_directory_datafolder, p.properties["output_jsonld"])
+    init_data.output_directory_datafolder = output_directory_datafolder
+    init_data.type = type
+    init_data.yaml_file = input_yaml
 
-    return data
+    # Check for not supported fields
+    for uninit_data in data.keys():
+        print(f"\nERROR: The field '{uninit_data}' is not supported or is misspelled. Check out our documentation https://github.com/oeg-upm/ya2ro/blob/main/doc.md")
+
+    return init_data
 
 
 def init_project(input_to_vocab, data):
@@ -205,21 +235,29 @@ def init_project(input_to_vocab, data):
 
     if project.title:
         print("    - Title: Done.")
+        del data["title"]
+
 
     if project.goal:
         print("    - Goal: Done.")
+        del data["goal"]
+        
 
     if project.social_motivation:
         print("    - Social motivation: Done.")
+        del data["social_motivation"]
 
     if project.sketch:
         print("    - Sketch: Done.")
-    
+        del data["sketch"]
+
     if project.areas:
         print("    - Areas: Done.")
+        del data["areas"]
     
     if project.activities:
         print("    - Activities: Done.")
+        del data["activities"]
 
     # Demo
     populate_demo(project, input_to_vocab, data)
@@ -244,7 +282,7 @@ def init_project(input_to_vocab, data):
 
 def init_paper(input_to_vocab, data):
 
-    # Create paper object and pupulate the lists with empty instances
+    # Create paper object and populate the lists with empty instances
     paper = Paper(
         title = _safe(input_to_vocab["title"], data),
         summary = _safe(input_to_vocab["summary"], data),
@@ -256,9 +294,12 @@ def init_paper(input_to_vocab, data):
 
     if paper.title:
         print("    - Title: Done.")
+        del data["title"]
 
     if paper.summary:
         print("    - Summary: Done.")
+        del data["summary"]
+
 
     # Datasets
     populate_datasets(paper, input_to_vocab, data)
@@ -276,50 +317,55 @@ def init_paper(input_to_vocab, data):
     doi_paper_link = _safe(input_to_vocab["doi_paper"], data)
     if doi_paper_link is not None:
         print(f"    - Fetching data from {doi_paper_link}.")
-        paper_bib = req_doi.bib(doi_paper_link)
 
-        if paper.title is None:
-            print(f"        + Fetching title from {doi_paper_link}.")
-            paper.title = paper_bib.get_title()
-        
-        if paper.summary is None:
-            print(f"        + Fetching summary from {doi_paper_link}.")
-            paper.summary = paper_bib.get_summary()
+        try:
+            paper_bib = req_doi.bib(doi_paper_link)
 
-        # Add bibtext
-        paper.bib = paper_bib.get_bibtext()
+            if paper.title is None:
+                print(f"        + Fetching title from {doi_paper_link}.")
+                paper.title = paper_bib.get_title()
+            
+            if paper.summary is None:
+                print(f"        + Fetching summary from {doi_paper_link}.")
+                paper.summary = paper_bib.get_summary()
 
-        # Add citation
-        citation = paper_bib.get_citation()
-        if citation:
+            # Add bibtext
+            paper.bib = paper_bib.get_bibtext()
 
-            if not paper.bibliography:
-                paper.bibliography = []
-                
-            paper.bibliography.append(
-                Bibliography_entry(entry = citation)
-                )
+            # Add citation
+            citation = paper_bib.get_citation()
+            if citation:
 
-        # Add authors with DOI
-        doi_authors = paper_bib.get_authors()
-        paper_authors = [ a.name for a in paper.authors ]
-        if doi_authors:
-            for doi_author in doi_authors:
-                if doi_author not in paper_authors:
+                if not paper.bibliography:
+                    paper.bibliography = []
+                    
+                paper.bibliography.append(
+                    Bibliography_entry(entry = citation)
+                    )
 
-                    print(f"        + Author: {doi_author} extracted from {doi_paper_link}.")
-                    print(f"        + Using default photo for {doi_author}.")
+            # Add authors with DOI
+            doi_authors = paper_bib.get_authors()
+            if doi_authors:
+                for doi_author in doi_authors:
+                    if not paper.authors or doi_author not in [ a.name for a in paper.authors ]:
 
-                    # TODO: Add more data for new authors
+                        print(f"        + Author: {doi_author} extracted from {doi_paper_link}.")
+                        print(f"        + Using default photo for {doi_author}.")
 
-                    if not paper.authors:
-                        paper.authors = []
+                        # TODO: Add more data for new authors
 
-                    paper.authors.append(Author(
-                        name = doi_author, 
-                        photo = Path(input_to_vocab["images"], p.properties["default_author_img"]),
-                        role= "Author"
-                    ))
+                        if not paper.authors:
+                            paper.authors = []
+
+                        paper.authors.append(Author(
+                            name = doi_author, 
+                            photo = Path(input_to_vocab["images"], p.properties["default_author_img"]),
+                            role= "Author"
+                        ))
+        except:
+            print(f"ERROR: doi_paper is not valid '{doi_paper_link}'")
+
+        del data["doi_paper"]
                     
     return paper
     
@@ -349,46 +395,98 @@ def populate_datasets(object, input_to_vocab, data):
     i = 0
     for dataset in data[input_to_vocab["datasets"]]:
 
-        doi_dataset = _safe(input_to_vocab["doi_dataset"], dataset)
-        if doi_dataset is not None:
-
-            object.datasets[i].doi_dataset = doi_dataset
-            object.datasets[i].link = doi_dataset
-
-            if str(doi_dataset).startswith("https://doi"):
-                print(f"        + Fetching dataset data from {doi_dataset}")
-                req_doi_dataset = req_doi.dataset(doi_dataset)
-                object.datasets[i].description = req_doi_dataset.get_description()
-                object.datasets[i].name = req_doi_dataset.get_name()
-                object.datasets[i].license = req_doi_dataset.get_license()
-                object.datasets[i].author = req_doi_dataset.get_author()
-
+        doi = None
+        path = None
+        link = None
+        
+        if isinstance(dataset, dict):
+            doi = _safe(input_to_vocab["doi"], dataset)
+            path = _safe(input_to_vocab["path"], dataset)
+            link = _safe(input_to_vocab["link"], dataset)
+        elif isinstance(dataset, str):
+            if 'doi' in dataset:
+                doi = dataset
             else:
-                print(f"ERROR: {doi_dataset} is not a DOI.")
-            
-        link = _safe(input_to_vocab["link"], dataset)
-        if link is not None:
-            object.datasets[i].link = link
+                link = dataset
+        else:
+            print(f"WARNING: {dataset} format is not supported, see documentation for further guidance.")
 
+        if doi:
+
+            object.datasets[i].doi = doi
+            object.datasets[i].link = doi
+
+            try:
+                req_d = req_doi.dataset(doi)
+                print(f"        + Fetching dataset data from {doi}")
+                object.datasets[i].description = req_d.get_description()
+                object.datasets[i].name = req_d.get_name()
+                object.datasets[i].license = req_d.get_license()
+                object.datasets[i].author = req_d.get_author()
+
+            except:
+                print(f"ERROR: {doi} is not a DOI. Use the field 'link' or 'path' instead.")
+
+        elif path:
+            object.datasets[i].path = path
+            object.datasets[i].files = []
+
+            # Create datasets directory inside output folder
+            datasets_output_path = Path(output_directory_datafolder, "datasets")
+            if not os.path.exists(datasets_output_path):
+                os.makedirs(datasets_output_path)
+                print(f"Creating datasets directory {datasets_output_path}")
+
+            if os.path.isdir(path):
+                directory = os.fsencode(path)
+                for file in os.listdir(directory):
+                    # copy dataset to output/images directory
+                    filename = os.fsdecode(file)
+                    src = Path(path,filename)
+                    dst = Path(datasets_output_path, filename)
+                    copyfile(src, dst)
+                    object.datasets[i].files.append(f"datasets/{filename}")
+            elif os.path.isfile(path):
+                # copy dataset to output/images directory
+                src = Path(path)
+                filename = os.path.basename(src)
+                dst = Path(datasets_output_path, filename)
+                copyfile(src, dst)
+                object.datasets[i].files.append(f"datasets/{filename}")
+            else:
+                print(f"ERROR: {path} does not exist.")
+
+        elif link:
+            object.datasets[i].link = link
+            try:
+                page = metadata_parser.MetadataParser(url=link)
+                print(f"        + Trying to extract metadata from {link}.")
+                object.datasets[i].description = page.get_metadata('description')
+                object.datasets[i].name = page.get_metadata('title')
+                object.datasets[i].license = page.get_metadata('license')
+            except:
+                print(f"WARNING: No metadata could be extracted from {link}")
+
+            
         name = _safe(input_to_vocab["name"], dataset)
-        if name is not None:
+        if name:
             object.datasets[i].name = name 
         
-        descripton = _safe(input_to_vocab["description"], dataset)
-        if descripton is not None:
-            object.datasets[i].description = descripton
+        description = _safe(input_to_vocab["description"], dataset)
+        if description:
+            object.datasets[i].description = description
         
         license = _safe(input_to_vocab["license"], dataset)
-        if license is not None:
+        if license:
             object.dataset[i].license = license
         
         author = _safe(input_to_vocab["author"], dataset)
-        if author is not None:
+        if author:
             object.dataset[i].author = author
-        
-
+    
         i += 1
     print("    - Datasets: Done.")
+    del data["datasets"]
 
 
 def populate_software(object, input_to_vocab, data):
@@ -398,28 +496,41 @@ def populate_software(object, input_to_vocab, data):
 
     i = 0
     for software in data[input_to_vocab["software"]]:
-        link = _safe(input_to_vocab["link"], software)
+
+        if isinstance(software, str) and 'http' in software:
+            link = software
+        else:
+            link = _safe(input_to_vocab["link"], software)
         
         if link is not None:
             object.software[i].link = link
 
-            if link.startswith("https://github.com/"):
+            if link.startswith("https://github.com/") and not p.no_somef:
 
                 print(f"        + Using SOMEF for {link}")
-                header = {}
-                header['accept'] = 'application/vnd.github.v3+json'
+                print("            - Downloading repository... This may take a while.")
 
                 with HiddenPrints():
-                    _, github_data = somef.cli.load_repository_metadata(link, header)
+                    metadata = cli_get_data(0.9, False, link)
+                
+                soca_meta = soca_metadata(output_directory_datafolder, metadata.results)
 
-                object.software[i].name = _safe("name", github_data)
-                object.software[i].description = _safe("description", github_data)
-                object.software[i].license = _safe("name",_safe("license", github_data))
+                object.software[i].metadata = metadata.results
+                object.software[i].name = soca_meta.title()
+                object.software[i].description = soca_meta.description()
+                license = soca_meta.license()
+                if license:
+                    license = license['name']
+                object.software[i].license = soca_meta.license()
 
         name = _safe(input_to_vocab["name"], software)
         if name is not None:
             object.software[i].name = name
         
+        if not object.software[i].name:
+            object.software[i].name = link
+            print(f'WARNING: No name given to software "{link}", using pointer as name.')
+        
         description = _safe(input_to_vocab["description"], software)
         if description is not None:
             object.software[i].description = description
@@ -430,6 +541,7 @@ def populate_software(object, input_to_vocab, data):
 
         i += 1
     print("    - Software: Done.")
+    del data["software"]
 
 
 def populate_demo(object, input_to_vocab, data):
@@ -455,6 +567,7 @@ def populate_demo(object, input_to_vocab, data):
         i += 1
         
     print("    - Demo: Done.")
+    del data["demo"]
 
 
 def populate_bibliography(object, input_to_vocab, data):
@@ -464,11 +577,16 @@ def populate_bibliography(object, input_to_vocab, data):
 
     i = 0
     for entry in data[input_to_vocab["bibliography"]]:
-
-        object.bibliography[i].entry = entry
+        try:
+            paper_bib = req_doi.bib(entry)
+            print(f"        + Fetching bibliography entry from {entry}.")
+            object.bibliography[i].entry = paper_bib.get_citation()
+        except:
+            object.bibliography[i].entry = entry
 
         i += 1
     print("    - Bibliography: Done.")
+    del data["bibliography"]
 
 
 def populate_contact(object, input_to_vocab, data):
@@ -488,6 +606,8 @@ def populate_contact(object, input_to_vocab, data):
         print("ERROR: Contact phone and/or email are not decalred.")
         exit()
 
+    del data["contact"]
+    
 
 def populate_authors(object, input_to_vocab, data, field_of_author = "authors"):
 
@@ -501,15 +621,22 @@ def populate_authors(object, input_to_vocab, data, field_of_author = "authors"):
         # Orcid implementation
         orcid_link = _safe(input_to_vocab["orcid"], author)
         if orcid_link is not None:
-            print(f"        + Fetching author data from {orcid_link}.")
-            orcid = req_orcid.orcid(orcid_link)
-            object.authors[i].orcid = orcid_link
-            name = orcid.get_full_name()
-            object.authors[i].name = name
-            object.authors[i].position = ", ".join(orcid.get_affiliation())
-            object.authors[i].web = orcid.get_webs()[-1]
-            object.authors[i].description = orcid.get_bio()
-        
+            try:
+                orcid = req_orcid.orcid(orcid_link)
+                print(f"        + Fetching author data from {orcid_link}.")
+                object.authors[i].orcid = orcid_link
+                name = orcid.get_full_name()
+                object.authors[i].name = name
+                aff = orcid.get_affiliation()
+                if aff:
+                    object.authors[i].position = ", ".join(aff)
+                webs = orcid.get_webs()
+                if webs:
+                    object.authors[i].web = webs[-1]
+                object.authors[i].description = orcid.get_bio()
+            except:
+                print(f"ERROR: ORCID is not valid or not up '{orcid_link}'")
+            
         aux = _safe(input_to_vocab["name"], author)
         name =  aux if aux is not None else name
         if name is not None:
@@ -542,13 +669,4 @@ def populate_authors(object, input_to_vocab, data, field_of_author = "authors"):
         
         i += 1
     print("    - Authors: Done.")
-
-
-class HiddenPrints:
-    def __enter__(self):
-        self._original_stdout = sys.stdout
-        sys.stdout = open(os.devnull, 'w')
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        sys.stdout.close()
-        sys.stdout = self._original_stdout
+    del data[field_of_author]
