@@ -4,6 +4,9 @@ from shutil import copyfile
 from pathlib import Path
 import ntpath
 from . import hilite_me
+from soca.commands.portal.card import html_view as card_html_view
+import os
+import re
 
 class ro_html(object):
 
@@ -95,7 +98,7 @@ class ro_html(object):
 
             if self.data.type == "project":
                 logo_recognition_explained =f"""
-                <p>This EELISA logo showcases that this project meets all the following requirements, so it is considered to be a complete/elegible EELISA project:</p>
+                <p>This EELISA logo showcases that this project meets all the following requirements, so it is considered to be a complete/eligible EELISA project:</p>
                 {self.ul_component(req.replace("_", " ").capitalize() for req in self.data.requirements)}
                 """
                 img_component =f"""
@@ -150,7 +153,6 @@ class ro_html(object):
 
         ro_html.append_component(self.soup_help, "how_info", how_info_component)
 
-
     def init_styles(self):
 
         if p.style == "dark":
@@ -165,7 +167,6 @@ class ro_html(object):
             ro_html.append_component(self.soup, "style", style_component)
             ro_html.append_component(self.soup_help, "style", style_component)
             
-
     def init_requirements_recognition(self, requirements):
 
         # LOGO WORTH IT?
@@ -173,7 +174,7 @@ class ro_html(object):
         for req in requirements:
             val = getattr(self.data, req)
             if val is None:
-                print("WARNING: '{}' is not defined. Add it to be eligible for beeing an EELISA project.".format(req))
+                print("WARNING: '{}' is not defined. Add it to be eligible for being an EELISA project.".format(req))
                 self.worth = False
 
         if self.worth and self.data.type == "project":
@@ -187,7 +188,7 @@ class ro_html(object):
         if self.worth and self.data.type == "paper":
 
             # copy image to output/images directory
-            src = Path("images","complete_paper.png")
+            src = Path(p.base_dir, p.input_to_vocab["images"],"complete_paper.png")
             dst = Path(self.data.output_directory_datafolder, "images","complete_paper.png")
             copyfile(src, dst)
 
@@ -208,7 +209,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "social_motivation", social_motivation_component)
         ro_html.sidebar_append(self.soup, "social_motivation", "Social motivation")
     
-
     def init_sketch(self, sketch):
         
         sketch_component = f"""<div class="w3-container" style="margin-top:15px">
@@ -224,7 +224,6 @@ class ro_html(object):
         src = Path(sketch)
         dst = Path(self.data.output_directory_datafolder, sketch)
         copyfile(src, dst)
-    
 
     def init_areas(self, areas):
 
@@ -239,7 +238,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "areas", areas_component)
         ro_html.sidebar_append(self.soup, "areas", "Areas")
 
-
     def init_contact(self, contact):
 
         contact_elements = []
@@ -257,7 +255,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "contact", contact_component)
         ro_html.sidebar_append(self.soup, "contact", "Contact")
         
-
     def init_activities(self, activities):
 
         activities_list = self.ul_component(activities)
@@ -271,7 +268,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "activities", activities_component)
         ro_html.sidebar_append(self.soup, "activities", "Activities")
 
-
     def init_demo(self, demo):
 
         demo_list_commponent = self.ul_component([f"""<a href="{d.link}">{d.link if d.name is None else d.name}</a>: {d.description}""" for d in demo])
@@ -285,7 +281,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "demo", demo_component)
         ro_html.sidebar_append(self.soup, "demo", "Demo")
 
-
     def init_goal(self, goal):
 
         goal_component = f"""<div class="w3-container" style="margin-top:15px">
@@ -297,7 +292,6 @@ class ro_html(object):
         ro_html.append_component(self.soup, "goal", goal_component)
         ro_html.sidebar_append(self.soup, "goal", "Goal")
  
-
     def init_title(self, title):
 
         # modify web title metadata
@@ -305,7 +299,6 @@ class ro_html(object):
         # create the title
         self.soup.find(id = "showcase").h1.string = title
     
-
     def init_summary(self, summary):
 
         summary_component = f"""<div class="w3-container" style="margin-top:15px">
@@ -317,23 +310,39 @@ class ro_html(object):
         ro_html.append_component(self.soup, "summary", summary_component)
         ro_html.sidebar_append(self.soup, "summary", "Summary")
 
-
     def init_datasets(self, datasets):
 
         def html_entry_datasets(d):
             dataset_attr = []
+            if d.path:
+                if d.description:
+                    dataset_attr.append(f"<b>Description:</b> {d.description}")
+                
+                if d.license:
+                    dataset_attr.append(f"<b>License:</b> {d.license}")
+                
+                if d.author:
+                    dataset_attr.append(f"<b>Author:</b> {d.author}")
+                
+                if d.files:
+                    dataset_attr.append("<p>Pointers to download datasets:</p>" +
+                        self.ul_component([ f"""<a href="{ds}" download>{os.path.basename(Path(ds))}</a>""" for ds in d.files ])
+                        )
 
-            if d.description:
-                dataset_attr.append(f"<b>Description:</b> {d.description}")
-            
-            if d.license:
-                dataset_attr.append(f"<b>License:</b> {d.license}")
-            
-            if d.author:
-                dataset_attr.append(f"<b>Author:</b> {d.author}")
+                return f"""<p>{d.path if d.name is None else d.name}</p>
+                {self.ul_component(dataset_attr)}"""
+            else:
+                if d.description:
+                    dataset_attr.append(f"<b>Description:</b> {d.description}")
+                
+                if d.license:
+                    dataset_attr.append(f"<b>License:</b> {d.license}")
+                
+                if d.author:
+                    dataset_attr.append(f"<b>Author:</b> {d.author}")
 
-            return f"""<p><a href="{d.link}">{d.link if d.name is None else d.name}</a></p>
-            {self.ul_component(dataset_attr)}"""
+                return f"""<p><a href="{d.link}">{d.link if d.name is None else d.name}</a></p>
+                {self.ul_component(dataset_attr)}"""
 
         dataset_entries = [ html_entry_datasets(d) for d in datasets ]
         datasets_list_commponent = self.ul_component(dataset_entries)
@@ -347,7 +356,6 @@ class ro_html(object):
         ro_html.sidebar_append(self.soup, "datasets", "Datasets")
         ro_html.append_component(self.soup, "datasets", datasets_component)
     
-
     def init_software(self, software):
 
         def html_entry_software(s):
@@ -362,23 +370,39 @@ class ro_html(object):
             return f"""<p><a href="{s.link}">{s.link if s.name is None else s.name}</a></p>
             {self.ul_component(software_attr)}"""
 
-        software_entries = [ html_entry_software(s) for s in software ]
-        software_list_commponent = self.ul_component(software_entries)
+        software_cards_soca = ''
+        software_cards_html = []
+        
+        for s in software:
+            if s.metadata and not p.no_somef:
+                software_cards_soca += card_html_view(self.data.output_directory_datafolder ,s.metadata,embedded=True)
+            else:
+                software_cards_html.append(html_entry_software(s))
+
+        software_cards_component = f"""
+            {self.ul_component(software_cards_html)}
+            <div style="max-width: 1600px;display: flex; align-content: flex-start; flex-wrap: wrap; 
+            flex-direction: row;justify-content: center; margin-top: 3rem;font-size: 15px;">{software_cards_soca}</div>"""
 
         software_component = f"""<div class="w3-container" id="software" style="margin-top:75px">
-		<h1 class="w3-xxxlarge w3-text-green"><b>Software</b></h1>
-		<hr style="width:50px;border:5px solid green" class="w3-round">
+        <h1 class="w3-xxxlarge w3-text-green"><b>Software</b></h1>
+        <hr style="width:50px;border:5px solid green" class="w3-round">
         The pointers for the main software used can be found below:
-        {software_list_commponent}
-	    </div>"""
+        {software_cards_component}
+        </div>"""
 
         ro_html.sidebar_append(self.soup, "software", "Software")
         ro_html.append_component(self.soup, "software", software_component)
     
-
     def init_bibliography(self, bibliography):
- 
-        bibliography_list = self.ul_component([b.entry for b in bibliography])
+        
+        def html_activate_links(text):
+            return re.sub(
+                pattern=r'(http[^ 	)(]*)', 
+                repl='<a href="\\1" target="_blank">\\1</a>', 
+                string=text
+            )
+        bibliography_list = self.ul_component([html_activate_links(b.entry) for b in bibliography])
 
         bibliography_commponent = f"""<div class="w3-container" id="bib" style="margin-top:75px">
 		<h1 class="w3-xxxlarge w3-text-green"><b>Bibliography</b></h1>
@@ -389,7 +413,6 @@ class ro_html(object):
         ro_html.sidebar_append(self.soup, "bibliography", "Bibliography")
         ro_html.append_component(self.soup, "bibliography", bibliography_commponent)
 
-
     def init_authors(self, authors):
         # create authors
         authors_boxes = self.create_about_authors(authors)
@@ -414,7 +437,6 @@ class ro_html(object):
 
             copyfile(src, dst)
 
-
     def create_about_authors(self, authors):
 
         num_authors = 0
@@ -459,9 +481,8 @@ class ro_html(object):
 
         return html_author
 
-
     def create_HTML_file(self):
-        """Dupms index.html and dependencies into specified folder."""
+        """Dumps index.html and dependencies into specified folder"""
         # dump changes into index.html
         with open(self.data.output_html, "w+") as file:
             file.write(str(self.soup))
@@ -473,33 +494,29 @@ class ro_html(object):
         print(f"HTML website file created at {self.data.output_html}")   
         print(f"HTML help website file created at {self.data.output_html_help}")   
 
-
     def append_component(soup, location_id, str_component):
         loc = soup.find(id=location_id)
         html_component = BeautifulSoup(str_component, 'html.parser')
         loc.append(html_component)
 
-
     def sidebar_append(soup, location_id, item_name):
         item_component = f"""<a href="#{location_id}" onclick="w3_close()" class="w3-bar-item w3-button w3-hover-white">{item_name}</a>"""
         ro_html.append_component(soup, "sidebar", item_component)
 
-
     def ul_component(self, list):
         ul_list = """<ul>"""
         for li in list:
             ul_list += f"""<li>{li}</li>"""
         ul_list += """</ul>"""
-        return ul_list
+        return ul_list 
     
-    
-    def html_horizontal(left_element, rigth_element):
+    def html_horizontal(left_element, right_element):
         return f"""	<div class="row" >
 		<div class="column" style="width:30%;">
         {left_element}
 		</div>
         <div class="column" style="width:70%;">
-        {rigth_element}
+        {right_element}
 		</div>
 	    </div>"""
 
